@@ -16,14 +16,13 @@
 from __future__ import print_function
 
 import collections
+import json
 import os
 import re
 import shutil
-import StringIO
 import sys
 import tempfile
 import textwrap
-import urlparse
 import uuid
 import warnings
 import zipfile
@@ -35,10 +34,12 @@ from oslo_utils import importutils
 import prettytable
 import requests
 import six
+from six.moves import urllib
 import yaml
 import yaql
 
 from muranoclient.common import exceptions
+from muranoclient.i18n import _
 
 try:
     import yaql.language  # noqa
@@ -74,7 +75,9 @@ def pretty_choice_list(l):
     return ', '.join("'%s'" % i for i in l)
 
 
-def print_list(objs, fields, field_labels, formatters={}, sortby=0):
+def print_list(objs, fields, field_labels, formatters=None, sortby=0):
+    if formatters is None:
+        formatters = {}
     pt = prettytable.PrettyTable([f for f in field_labels], caching=False)
     pt.align = 'l'
 
@@ -90,7 +93,9 @@ def print_list(objs, fields, field_labels, formatters={}, sortby=0):
     print(encodeutils.safe_encode(pt.get_string()))
 
 
-def print_dict(d, formatters={}):
+def print_dict(d, formatters=None):
+    if formatters is None:
+        formatters = {}
     pt = prettytable.PrettyTable(['Property', 'Value'], caching=False)
     pt.align = 'l'
 
@@ -194,10 +199,12 @@ class NoCloseProxy(object):
 
 
 class File(object):
-    def __init__(self, name):
+    def __init__(self, name, binary=True):
         self.name = name
+        self.binary = binary
 
     def open(self):
+        mode = 'rb' if self.binary else 'r'
         if hasattr(self.name, 'read'):
             # NOTE(kzaitsev) We do not want to close a file object
             # passed to File wrapper. The caller should be responsible
@@ -205,25 +212,24 @@ class File(object):
             return NoCloseProxy(self.name)
         else:
             if os.path.isfile(self.name):
-                return open(self.name)
-            url = urlparse.urlparse(self.name)
+                return open(self.name, mode)
+            url = urllib.parse.urlparse(self.name)
             if url.scheme in ('http', 'https'):
                 resp = requests.get(self.name, stream=True)
                 if not resp.ok:
                     raise ValueError("Got non-ok status({0}) "
                                      "while connecting to {1}".format(
                                          resp.status_code, self.name))
-                temp_file = tempfile.NamedTemporaryFile()
+                temp_file = tempfile.NamedTemporaryFile(mode='w+b')
                 for chunk in resp.iter_content(1024 * 1024):
                     temp_file.write(chunk)
                 temp_file.flush()
-                temp_file.seek(0)
-                return temp_file
+                return open(temp_file.name, mode)
             raise ValueError("Can't open {0}".format(self.name))
 
 
 def to_url(filename, base_url, version='', path='/', extension=''):
-    if urlparse.urlparse(filename).scheme in ('http', 'https'):
+    if urllib.parse.urlparse(filename).scheme in ('http', 'https'):
         return filename
     if not base_url:
         raise ValueError("No base_url for repository supplied")
@@ -231,7 +237,8 @@ def to_url(filename, base_url, version='', path='/', extension=''):
         raise ValueError("Invalid filename path supplied: {0}".format(
             filename))
     version = '.' + version if version else ''
-    return urlparse.urljoin(base_url, path + filename + version + extension)
+    return urllib.parse.urljoin(base_url,
+                                path + filename + version + extension)
 
 
 class FileWrapperMixin(object):
@@ -252,15 +259,16 @@ class FileWrapperMixin(object):
         if self._file and not self._file.closed:
             self._file.close()
 
-    def save(self, dst):
+    def save(self, dst, binary=True):
         file_name = self.file_wrapper.name
 
-        if urlparse.urlparse(file_name).scheme:
+        if urllib.parse.urlparse(file_name).scheme:
             file_name = file_name.split('/')[-1]
 
         dst = os.path.join(dst, file_name)
 
-        with open(dst, 'wb') as dst_file:
+        mode = 'wb' if binary else 'w'
+        with open(dst, mode) as dst_file:
             self._file.seek(0)
             shutil.copyfileobj(self._file, dst_file)
 
@@ -304,7 +312,7 @@ class Package(FileWrapperMixin):
             name,
             base_url=base_url,
             version=version,
-            path='/apps/',
+            path='apps/',
             extension='.zip')
         )
 
@@ -315,7 +323,7 @@ class Package(FileWrapperMixin):
             try:
                 self._file.seek(0)
                 self._zip_obj = zipfile.ZipFile(
-                    StringIO.StringIO(self._file.read()))
+                    six.BytesIO(self._file.read()))
             except Exception as e:
                 LOG.error("Error {0} occurred,"
                           " while parsing the package".format(e))
@@ -385,7 +393,7 @@ class Package(FileWrapperMixin):
             dep_dict = {}
         dep_dict[self.manifest['FullName']] = self
         if 'Require' in self.manifest:
-            for dep_name, ver in self.manifest['Require'].iteritems():
+            for dep_name, ver in six.iteritems(self.manifest['Require']):
                 if dep_name in dep_dict:
                     continue
                 try:
@@ -396,7 +404,7 @@ class Package(FileWrapperMixin):
                         base_url=base_url,
                     )
                 except Exception as e:
-                    LOG.error("Error {0} occured while parsing package {1}, "
+                    LOG.error("Error {0} occurred while parsing package {1}, "
                               "required by {2} package".format(
                                   e, dep_name,
                                   self.manifest['FullName']))
@@ -415,10 +423,10 @@ def save_image_local(image_spec, base_url, dst):
     download_url = to_url(
         image_spec.get("Url", image_spec['Name']),
         base_url=base_url,
-        path='/images/'
+        path='images/'
     )
 
-    with open(dst, "wb") as image_file:
+    with open(dst, "w") as image_file:
         response = requests.get(download_url, stream=True)
         total_length = response.headers.get('content-length')
 
@@ -438,7 +446,9 @@ def save_image_local(image_spec, base_url, dst):
             image_file.flush()
 
 
-def ensure_images(glance_client, image_specs, base_url, local_path=None):
+def ensure_images(glance_client, image_specs, base_url,
+                  local_path=None,
+                  is_package_public=False):
     """Ensure that images from image_specs are available in glance. If not
     attempts: instructs glance to download the images and sets murano-specific
     metadata for it.
@@ -464,7 +474,7 @@ def ensure_images(glance_client, image_specs, base_url, local_path=None):
 
         images = glance_client.images.list(filters=filters)
         try:
-            img = images.next().to_dict()
+            img = next(images).to_dict()
         except StopIteration:
             img = None
 
@@ -503,7 +513,7 @@ def ensure_images(glance_client, image_specs, base_url, local_path=None):
                 download_url = to_url(
                     image_spec.get("Url", image_spec['Name']),
                     base_url=base_url,
-                    path='/images/',
+                    path='images/',
                 )
                 LOG.info("Instructing glance to download image {0}".format(
                     image_spec['Name']))
@@ -513,6 +523,15 @@ def ensure_images(glance_client, image_specs, base_url, local_path=None):
                     disk_format=image_spec['DiskFormat'],
                     copy_from=download_url)
                 img = img.to_dict()
+
+            if is_package_public:
+                try:
+                    glance_client.images.update(img['id'], is_public=True)
+                    LOG.debug('Success update for image {0}'.format(img['id']))
+                except Exception as e:
+                    LOG.exception(_("Error {0} occurred while setting "
+                                    "image {1} public").format(e, img['id']))
+
             installed_images.append(img)
 
             if update_metadata and 'Meta' in image_spec:
@@ -531,7 +550,7 @@ class Bundle(FileWrapperMixin):
     @staticmethod
     def from_file(file_obj):
         if not isinstance(file_obj, File):
-            file_obj = File(file_obj)
+            file_obj = File(file_obj, binary=False)
         return Bundle(file_obj)
 
     @staticmethod
@@ -540,13 +559,15 @@ class Bundle(FileWrapperMixin):
         return Bundle.from_file(file_obj)
 
     def package_specs(self):
-        """Returns a generator yeilding package specifications i.e.
+        """Returns a generator yielding package specifications i.e.
         dicts with 'Name' and 'Version' fields
         """
         self._file.seek(0)
         bundle = None
         try:
-            bundle = jsonutils.load(self._file)
+            # NOTE(kzaitsev) jsonutils throws a type error here
+            # see bug 1515231
+            bundle = json.load(self._file)
         except ValueError:
             pass
         if bundle is None:
@@ -555,12 +576,9 @@ class Bundle(FileWrapperMixin):
             except yaml.error.YAMLError:
                 pass
 
-        if bundle is None:
+        if bundle is None or 'Packages' not in bundle:
             raise ValueError("Can't parse bundle contents")
 
-        if 'Packages' not in bundle:
-            return
-
         for package in bundle['Packages']:
             if 'Name' not in package:
                 continue
@@ -581,7 +599,7 @@ class Bundle(FileWrapperMixin):
                 )
 
             except Exception as e:
-                LOG.error("Error {0} occured while obtaining "
+                LOG.error("Error {0} occurred while obtaining "
                           "package {1}".format(e, package['Name']))
                 continue
             yield pkg_obj
@@ -601,8 +619,8 @@ def yaql_constructor(loader, node):
     value = loader.construct_scalar(node)
     return YaqlExpression(value)
 
-yaml.add_constructor(u'!yaql', yaql_constructor, YaqlYamlLoader)
-yaml.add_implicit_resolver(u'!yaql', YaqlExpression, Loader=YaqlYamlLoader)
+YaqlYamlLoader.add_constructor(u'!yaql', yaql_constructor)
+YaqlYamlLoader.add_implicit_resolver(u'!yaql', YaqlExpression, None)
 
 
 def traverse_and_replace(obj,
