@@ -17,11 +17,14 @@ import json
 import os
 import shutil
 import sys
+import tempfile
 import uuid
 import zipfile
 
 import jsonpatch
 from oslo_utils import strutils
+import six
+import six.moves
 
 from muranoclient.common import exceptions as common_exceptions
 from muranoclient.common import utils
@@ -32,16 +35,24 @@ from muranoclient.v1.package_creator import mpl_package
 _bool_from_str_strict = functools.partial(
     strutils.bool_from_string, strict=True)
 
+DEFAULT_PAGE_SIZE = 20
+
 
 @utils.arg('--all-tenants', action='store_true', default=False,
            help='Allows to list environments from all tenants'
                 ' (admin only).')
-def do_environment_list(mc, args={}):
+def do_environment_list(mc, args=None):
     """List the environments."""
+    if args is None:
+        args = {}
     all_tenants = getattr(args, 'all_tenants', False)
     environments = mc.environments.list(all_tenants)
-    field_labels = ['ID', 'Name', 'Created', 'Updated']
-    fields = ['id', 'name', 'created', 'updated']
+    _print_environment_list(environments)
+
+
+def _print_environment_list(environments):
+    field_labels = ['ID', 'Name', 'Status', 'Created', 'Updated']
+    fields = ['id', 'name', 'status', 'created', 'updated']
     utils.print_list(environments, fields, field_labels, sortby=0)
 
 
@@ -69,16 +80,18 @@ def _generate_join_existing_net(net, subnet):
            help="Network id to join.",)
 @utils.arg("--join-subnet-id", metavar="<SUBNET_ID>",
            help="Subnetwork id to join.",)
+@utils.arg("--region", metavar="<REGION_NAME>",
+           help="Name of the target OpenStack region.",)
 @utils.arg("name", metavar="<ENVIRONMENT_NAME>",
            help="Environment name.")
 def do_environment_create(mc, args):
     """Create an environment."""
-    body = {"name": args.name}
+    body = {"name": args.name, "region": args.region}
     if args.join_net_id or args.join_subnet_id:
         body.update(_generate_join_existing_net(
             args.join_net_id, args.join_subnet_id))
-    mc.environments.create(body)
-    do_environment_list(mc)
+    environment = mc.environments.create(body)
+    _print_environment_list([environment])
 
 
 @utils.arg("id", metavar="<NAME or ID>",
@@ -112,11 +125,11 @@ def do_environment_rename(mc, args):
     """Rename an environment."""
     try:
         environment = utils.find_resource(mc.environments, args.id)
-        mc.environments.update(environment.id, args.name)
+        environment = mc.environments.update(environment.id, args.name)
     except exceptions.NotFound:
         raise exceptions.CommandError("Environment %s not found" % args.id)
     else:
-        do_environment_list(mc)
+        _print_environment_list([environment])
 
 
 @utils.arg("id", metavar="<NAME or ID>",
@@ -192,7 +205,7 @@ def do_environment_action_call(mc, args):
            help="ID of action to run.")
 def do_environment_action_get_result(mc, args):
     """Get result of `TASK` in environment `ID`."""
-    result = mc.actions.call(args.id, args.task_id)
+    result = mc.actions.get_result(args.id, args.task_id)
     print("Task id result: {0}".format(result))
 
 
@@ -201,7 +214,9 @@ def do_environment_session_create(mc, args):
     """Creates a new configuration session for environment ID."""
     environment_id = args.id
     session_id = mc.sessions.configure(environment_id).id
-    print("Created new session: {0}".format(session_id))
+    print("Created new session:")
+    formatters = {"id": utils.text_wrap_formatter}
+    utils.print_dict({"id": session_id}, formatters=formatters)
 
 
 @utils.arg("id", metavar="<ID>", help="ID of Environment to edit.")
@@ -254,20 +269,48 @@ def do_environment_apps_edit(mc, args):
         session_id=session_id)
 
 
-def do_env_template_list(mc, args={}):
+def do_env_template_list(mc, args=None):
     """List the environments templates."""
+    if args is None:
+        args = {}
     env_templates = mc.env_templates.list()
-    field_labels = ['ID', 'Name', 'Created', 'Updated']
-    fields = ['id', 'name', 'created', 'updated']
+    _print_env_template_list(env_templates)
+
+
+def _print_env_template_list(env_templates):
+    field_labels = ['ID', 'Name', 'Created', 'Updated', 'Is public']
+    fields = ['id', 'name', 'created', 'updated', 'is_public']
     utils.print_list(env_templates, fields, field_labels, sortby=0)
 
 
 @utils.arg("name", metavar="<ENV_TEMPLATE_NAME>",
            help="Environment template name.")
+@utils.arg("--is-public", action='store_true', default=False,
+           help='Make the template available for users from other tenants.')
 def do_env_template_create(mc, args):
     """Create an environment template."""
-    mc.env_templates.create({"name": args.name})
-    do_env_template_list(mc)
+    env_template = mc.env_templates.create(
+        {"name": args.name, "is_public": args.is_public})
+    _print_env_template_list([env_template])
+
+
+@utils.arg("id", metavar="<ID>",
+           help="Environment template ID.")
+@utils.arg("name", metavar="<ENV_TEMPLATE_NAME>",
+           help="New environment name.")
+def do_env_template_create_env(mc, args):
+    """Create a new environment from template."""
+    try:
+        template = mc.env_templates.create_env(args.id, args.name)
+    except common_exceptions.HTTPNotFound:
+        raise exceptions.CommandError("Environment template %s not found"
+                                      % args.id)
+    else:
+        formatters = {
+            "environment_id": utils.text_wrap_formatter,
+            "session_id": utils.text_wrap_formatter
+        }
+        utils.print_dict(template.to_dict(), formatters=formatters)
 
 
 @utils.arg("id", metavar="<ID>",
@@ -276,7 +319,7 @@ def do_env_template_show(mc, args):
     """Display environment template details."""
     try:
         env_template = mc.env_templates.get(args.id)
-    except exceptions.NotFound:
+    except common_exceptions.HTTPNotFound:
         raise exceptions.CommandError("Environment template %s not found"
                                       % args.id)
     else:
@@ -291,16 +334,16 @@ def do_env_template_show(mc, args):
         utils.print_dict(env_template.to_dict(), formatters=formatters)
 
 
-@utils.arg("name", metavar="<ENV_TEMPLATE_NAME>",
-           help="Environment template name.")
+@utils.arg("id", metavar="<ENV_TEMPLATE_ID>",
+           help="Environment template ID.")
 @utils.arg('app_template_file', metavar='<FILE>',
            help='Path to the template.')
 def do_env_template_add_app(mc, args):
     """Add application to the environment template."""
-    with open(args.app_template_file, "r") as myfile:
-        app_file = myfile.readlines()
-    mc.env_templates.create_app(args.name, app_file)
-    do_env_template_list(mc)
+    with open(args.app_template_file, "r") as app_file:
+        app_template = json.load(app_file)
+    mc.env_templates.create_app(args.id, app_template)
+    do_env_template_show(mc, args)
 
 
 @utils.arg("id", metavar="<ENV_TEMPLATE_ID>",
@@ -309,8 +352,8 @@ def do_env_template_add_app(mc, args):
            help="Application ID.")
 def do_env_template_del_app(mc, args):
     """Delete application from the environment template."""
-    mc.env_templates.delete_app(args.name, args.app_id)
-    do_env_template_list(mc)
+    mc.env_templates.delete_app(args.id, args.app_id)
+    do_env_template_show(mc, args)
 
 
 @utils.arg("id", metavar="<ID>",
@@ -319,8 +362,8 @@ def do_env_template_del_app(mc, args):
            help="Environment template name.")
 def do_env_template_update(mc, args):
     """Update an environment template."""
-    mc.env_templates.update(args.id, args.name)
-    do_env_template_list(mc)
+    env_template = mc.env_templates.update(args.id, args.name)
+    _print_env_template_list([env_template])
 
 
 @utils.arg("id", metavar="<ID>",
@@ -331,7 +374,7 @@ def do_env_template_delete(mc, args):
     for env_template_id in args.id:
         try:
             mc.env_templates.delete(env_template_id)
-        except exceptions.NotFound:
+        except common_exceptions.HTTPNotFound:
             failure_count += 1
             mns = "Failed to delete '{0}'; environment template not found".\
                 format(env_template_id)
@@ -341,6 +384,32 @@ def do_env_template_delete(mc, args):
     do_env_template_list(mc)
 
 
+@utils.arg("id", metavar="<ID>",
+           help="Environment template ID.")
+@utils.arg("name", metavar="<ENV_TEMPLATE_NAME>",
+           help="New environment template name.")
+def do_env_template_clone(mc, args):
+    """Create a new template, cloned from template."""
+    try:
+        env_template = mc.env_templates.clone(args.id, args.name)
+    except common_exceptions.HTTPNotFound:
+        raise exceptions.CommandError("Environment template %s not found"
+                                      % args.id)
+    else:
+        formatters = {
+            "id": utils.text_wrap_formatter,
+            "created": utils.text_wrap_formatter,
+            "updated": utils.text_wrap_formatter,
+            "version": utils.text_wrap_formatter,
+            "name": utils.text_wrap_formatter,
+            "tenant_id": utils.text_wrap_formatter,
+            "is_public": utils.text_wrap_formatter,
+            "services": utils.json_formatter,
+
+        }
+        utils.print_dict(env_template.to_dict(), formatters=formatters)
+
+
 @utils.arg("id", metavar="<ID>",
            help="Environment ID for which to list deployments.")
 def do_deployment_list(mc, args):
@@ -356,22 +425,73 @@ def do_deployment_list(mc, args):
         utils.print_list(deployments, fields, field_labels, sortby=0)
 
 
+@utils.arg("--limit", type=int, default=DEFAULT_PAGE_SIZE)
 @utils.arg("--include-disabled", default=False, action="store_true")
-def do_package_list(mc, args={}):
+@utils.arg("--owned", default=False, action="store_true")
+@utils.arg('--search', metavar='<SEARCH_KEYS>',
+           dest='search', required=False,
+           help='Show packages, that match search keys fuzzily')
+@utils.arg('--name', metavar='<PACKAGE_NAME>',
+           dest='name', required=False,
+           help='Show packages, whose name match parameter exactly')
+@utils.arg('--fqn', metavar="<PACKAGE_FULLY_QUALIFIED_NAME>",
+           dest='fqn', required=False,
+           help='Show packages, '
+                'whose fully qualified name match parameter exactly')
+@utils.arg('--type', metavar='<PACKAGE_TYPE>',
+           dest='type', required=False,
+           help='Show packages, whose type match parameter exactly')
+@utils.arg('--category', metavar='<PACKAGE_CATEGORY>',
+           dest='category', required=False,
+           help='Show packages, whose categories include parameter')
+@utils.arg('--class_name', metavar='<PACKAGE_CLASS_NAME>',
+           dest='class_name', required=False,
+           help='Show packages, whose class name match parameter exactly')
+@utils.arg('--tag', metavar='<PACKAGE_TAG>',
+           dest='tag', required=False,
+           help='Show packages, whose tags include parameter')
+def do_package_list(mc, args=None):
     """List available packages."""
+    if args is None:
+        args = {}
     filter_args = {
         "include_disabled": getattr(args, 'include_disabled', False),
+        "limit": getattr(args, 'limit', DEFAULT_PAGE_SIZE),
+        "owned": getattr(args, 'owned', False),
     }
+    if args:
+        if args.search:
+            filter_args['search'] = args.search
+        if args.name:
+            filter_args['name'] = args.name
+        if args.fqn:
+            filter_args['fqn'] = args.fqn
+        if args.type:
+            filter_args['type'] = args.type
+        if args.category:
+            filter_args['category'] = args.category
+        if args.class_name:
+            filter_args['class_name'] = args.class_name
+        if args.tag:
+            filter_args['tag'] = args.tag
+
     packages = mc.packages.filter(**filter_args)
-    field_labels = ["ID", "Name", "FQN", "Author", "Is Public"]
-    fields = ["id", "name", "fully_qualified_name", "author", "is_public"]
+    _print_package_list(packages)
+
+
+def _print_package_list(packages):
+    field_labels = ["ID", "Name", "FQN", "Author", "Active",
+                    "Is Public", "Type"]
+    fields = ["id", "name", "fully_qualified_name", "author",
+              "enabled", "is_public", "type"]
     utils.print_list(packages, fields, field_labels, sortby=0)
 
 
 @utils.arg("id", metavar="<ID>",
            help="Package ID to download.")
 @utils.arg("filename", metavar="file", nargs="?",
-           help="Filename for download (defaults to stdout).")
+           help="Filename to save package to. If it is not specified and "
+                "there is no stdout redirection the package won't be saved.")
 def do_package_download(mc, args):
     """Download a package to a filename or stdout."""
 
@@ -379,13 +499,18 @@ def do_package_download(mc, args):
         fh.write(mc.packages.download(package_id))
 
     try:
-        if not args.filename:
-            download_to_fh(args.id, sys.stdout)
-        else:
+        if args.filename:
             with open(args.filename, 'wb') as fh:
                 download_to_fh(args.id, fh)
                 print("Package downloaded to %s" % args.filename)
-    except exceptions.NotFound:
+        elif not sys.stdout.isatty():
+            download_to_fh(args.id, sys.stdout)
+        else:
+            msg = ('No stdout redirection or local file specified for '
+                   'downloaded package. Please specify a local file to save '
+                   'downloaded package or redirect output to another source.')
+            raise exceptions.CommandError(msg)
+    except common_exceptions.HTTPNotFound:
         raise exceptions.CommandError("Package %s not found" % args.id)
 
 
@@ -395,7 +520,7 @@ def do_package_show(mc, args):
     """Display details for a package."""
     try:
         package = mc.packages.get(args.id)
-    except exceptions.NotFound:
+    except common_exceptions.HTTPNotFound:
         raise exceptions.CommandError("Package %s not found" % args.id)
     else:
         to_display = dict(
@@ -428,9 +553,8 @@ def do_package_delete(mc, args):
     for package_id in args.id:
         try:
             mc.packages.delete(package_id)
-            print("Deleted package  '{0}'".format(package_id))
+            print("Deleted package '{0}'".format(package_id))
         except exceptions.NotFound:
-            raise exceptions.CommandError("Package %s not found" % package_id)
             failure_count += 1
             print("Failed to delete '{0}'; package not found".
                   format(package_id))
@@ -444,6 +568,7 @@ def do_package_delete(mc, args):
 
 def _handle_package_exists(mc, data, package, exists_action):
     name = package.manifest['FullName']
+    version = package.manifest.get('Version', '0')
     while True:
         print("Importing package {0}".format(name))
         try:
@@ -456,17 +581,18 @@ def _handle_package_exists(mc, data, package, exists_action):
             if not res:
                 while True:
                     print("What do you want to do? (s)kip, (u)pdate, (a)bort")
-                    res = raw_input()
+                    res = six.moves.input()
                     if res in allowed_results:
                         break
             if res == 's':
                 print("Skipping.")
-                return
+                return None
             elif res == 'a':
                 print("Exiting.")
                 sys.exit()
             elif res == 'u':
-                pkgs = list(mc.packages.filter(fqn=name, owned=True))
+                pkgs = list(mc.packages.filter(fqn=name, version=version,
+                                               owned=True))
                 if not pkgs:
                     msg = (
                         "Got a conflict response, but could not find the "
@@ -493,7 +619,7 @@ def _handle_package_exists(mc, data, package, exists_action):
            help='Category list to attach.')
 @utils.arg('--is-public', action='store_true', default=False,
            help='Make the package available for users from other tenants.')
-@utils.arg('--version', default='',
+@utils.arg('--package-version', default='',
            help='Version of the package to use from repository '
                 '(ignored when importing with multiple packages).')
 @utils.arg('--exists-action', default='', choices=['a', 's', 'u'],
@@ -508,7 +634,7 @@ def do_package_import(mc, args):
     """
     data = {"is_public": args.is_public}
 
-    version = args.version
+    version = args.package_version
     if version and len(args.filename) >= 2:
         print("Requested to import more than one package, "
               "ignoring version.")
@@ -517,8 +643,6 @@ def do_package_import(mc, args):
     if args.categories:
         data["categories"] = args.categories
 
-    should_do_list = False
-
     total_reqs = {}
     for filename in args.filename:
         if os.path.isfile(filename):
@@ -539,10 +663,11 @@ def do_package_import(mc, args):
             print("Failed to create package for '{0}', reason: {1}".format(
                 filename, e))
             continue
-        should_do_list = True
         total_reqs.update(package.requirements(base_url=args.murano_repo_url))
 
-    for name, package in total_reqs.iteritems():
+    imported_list = []
+
+    for name, package in six.iteritems(total_reqs):
         image_specs = package.images()
         if image_specs:
             print("Inspecting required images")
@@ -550,7 +675,8 @@ def do_package_import(mc, args):
                 imgs = utils.ensure_images(
                     glance_client=mc.glance_client,
                     image_specs=image_specs,
-                    base_url=args.murano_repo_url)
+                    base_url=args.murano_repo_url,
+                    is_package_public=args.is_public)
                 for img in imgs:
                     print("Added {0}, {1} image".format(
                         img['name'], img['id']))
@@ -558,13 +684,16 @@ def do_package_import(mc, args):
                 print("Error {0} occurred while installing "
                       "images for {1}".format(e, name))
         try:
-            _handle_package_exists(mc, data, package, args.exists_action)
+            imported_package = _handle_package_exists(
+                mc, data, package, args.exists_action)
+            if imported_package:
+                imported_list.append(imported_package)
         except Exception as e:
             print("Error {0} occurred while installing package {1}".format(
                 e, name))
 
-    if should_do_list:
-        do_package_list(mc)
+    if imported_list:
+        _print_package_list(imported_list)
 
 
 @utils.arg("id", metavar="<ID>",
@@ -579,7 +708,7 @@ def do_package_import(mc, args):
            default=None,
            help='A list of keywords connected to the application.')
 def do_package_update(mc, args):
-    """Edit the package in question"""
+    """Update an existing package."""
     data = {}
     parameters = ('is_public', 'enabled',
                   'name', 'description',
@@ -607,7 +736,6 @@ def do_bundle_import(mc, args):
     file names, relative to location of the bundle file. Requirements
     are first searched in the same directory.
     """
-    should_do_list = False
     total_reqs = {}
     for filename in args.filename:
         local_path = None
@@ -620,7 +748,7 @@ def do_bundle_import(mc, args):
             _file = utils.to_url(
                 filename,
                 base_url=args.murano_repo_url,
-                path='/bundles/',
+                path='bundles/',
                 extension='.bundle',
             )
 
@@ -631,7 +759,6 @@ def do_bundle_import(mc, args):
                 filename, e))
             continue
 
-        should_do_list = True
         data = {"is_public": args.is_public}
 
         for package in bundle_file.packages(
@@ -643,7 +770,9 @@ def do_bundle_import(mc, args):
             )
             total_reqs.update(requirements)
 
-    for name, dep_package in total_reqs.iteritems():
+    imported_list = []
+
+    for name, dep_package in six.iteritems(total_reqs):
         image_specs = dep_package.images()
         if image_specs:
             print("Inspecting required images")
@@ -652,7 +781,8 @@ def do_bundle_import(mc, args):
                     glance_client=mc.glance_client,
                     image_specs=image_specs,
                     base_url=args.murano_repo_url,
-                    local_path=local_path)
+                    local_path=local_path,
+                    is_package_public=args.is_public)
                 for img in imgs:
                     print("Added {0}, {1} image".format(
                         img['name'], img['id']))
@@ -660,21 +790,23 @@ def do_bundle_import(mc, args):
                 print("Error {0} occurred while installing "
                       "images for {1}".format(e, name))
         try:
-            _handle_package_exists(
+            imported_package = _handle_package_exists(
                 mc, data, dep_package, args.exists_action)
+            if imported_package:
+                imported_list.append(imported_package)
         except exceptions.CommandError:
             raise
         except Exception as e:
             print("Error {0} occurred while "
                   "installing package {1}".format(e, name))
-    if should_do_list:
-        do_package_list(mc)
+    if imported_list:
+        _print_package_list(imported_list)
 
 
 def _handle_save_packages(packages, dst, base_url, no_images):
     downloaded_images = []
 
-    for name, pkg in packages.iteritems():
+    for name, pkg in six.iteritems(packages):
         if not no_images:
             image_specs = pkg.images()
             for image_spec in image_specs:
@@ -733,7 +865,7 @@ def do_bundle_save(mc, args):
         _file = utils.to_url(
             bundle,
             base_url=base_url,
-            path='/bundles/',
+            path='bundles/',
             extension='.bundle',
         )
     try:
@@ -751,7 +883,7 @@ def do_bundle_save(mc, args):
     _handle_save_packages(total_reqs, dst, base_url, no_images)
 
     try:
-        bundle_file.save(dst)
+        bundle_file.save(dst, binary=False)
         print("Bundle file {0} has been successfully saved".format(bundle))
     except Exception as e:
         print("Error {0} occurred while saving bundle {1}".format(e, bundle))
@@ -763,7 +895,7 @@ def do_bundle_save(mc, args):
 @utils.arg('-p', '--path', metavar='<PATH>',
            help='Path to the directory to store package. If not set will use '
                 'current directory.')
-@utils.arg('--version', default='',
+@utils.arg('--package-version', default='',
            help='Version of the package to use from repository '
                 '(ignored when saving with multiple packages).')
 @utils.arg('--no-images', action='store_true', default=False,
@@ -782,7 +914,7 @@ def do_package_save(mc, args):
     else:
         dst = os.getcwd()
 
-    version = args.version
+    version = args.package_version
     if version and len(args.filename) >= 2:
         print("Requested to save more than one package, "
               "ignoring version.")
@@ -801,7 +933,7 @@ def do_package_save(mc, args):
             pkg = utils.Package.from_file(_file)
         except Exception as e:
             print("Failed to create package for '{0}', reason: {1}".format(
-                pkg, e))
+                package, e))
             continue
         total_reqs.update(pkg.requirements(base_url=base_url))
 
@@ -821,21 +953,25 @@ def do_app_show(mc, args):
     """
     if args.path == '/':
         apps = mc.services.list(args.id)
+        formatters = {'id': lambda x: getattr(x, '?')['id'],
+                      'type': lambda x: getattr(x, '?')['type']}
+        field_labels = ['Id', 'Name', 'Type']
+        fields = ['id', 'name', 'type']
+        utils.print_list(apps, fields, field_labels, formatters=formatters)
     else:
         if not args.path.startswith('/'):
             args.path = '/' + args.path
-            apps = [mc.services.get(args.id, args.path)]
+        app = mc.services.get(args.id, args.path)
 
-    field_labels = ['Id', 'Name', 'Type']
-    fields = ['id', 'name', 'type']
-    formatters = {}
-
-    # If app list is empty, first element exists and it's None
-    if hasattr(apps[0], '?'):
-        formatters = {'id': lambda x: getattr(x, '?')['id'],
-                      'type': lambda x: getattr(x, '?')['type']}
-
-    utils.print_list(apps, fields, field_labels, formatters=formatters)
+        # If app with specified path is not found, it is empty.
+        if hasattr(app, '?'):
+            formatters = {}
+            for key in app.to_dict().keys():
+                formatters[key] = utils.json_formatter
+            utils.print_dict(app.to_dict(), formatters)
+        else:
+            raise exceptions.CommandError("Could not find application at path"
+                                          " %s" % args.path)
 
 
 @utils.arg('-t', '--template', metavar='<HEAT_TEMPLATE>',
@@ -870,19 +1006,20 @@ def do_package_create(mc, args):
     if not args.template and not args.classes_dir:
         raise exceptions.CommandError(
             "Provide --template for a HOT-based package, OR at least"
-            "--classes-dir for a MuranoPL-based package")
+            " --classes-dir for a MuranoPL-based package")
     directory_path = None
     try:
+        archive_name = args.output if args.output else None
         if args.template:
             directory_path = hot_package.prepare_package(args)
+            if not archive_name:
+                archive_name = os.path.basename(args.template)
+                archive_name = os.path.splitext(archive_name)[0] + ".zip"
         else:
             directory_path = mpl_package.prepare_package(args)
-
-        if args.output:
-            archive_name = args.output
-        else:
-            archive_name = os.path.splitext(os.path.basename(args.template))[0]
-            archive_name += ".zip"
+            if not archive_name:
+                archive_name = tempfile.mkstemp(
+                    prefix="murano_", dir=os.getcwd())[1] + ".zip"
 
         _make_archive(archive_name, directory_path)
         print("Application package is available at " +
@@ -901,9 +1038,15 @@ def _make_archive(archive_name, path):
                                                 f))
 
 
-def do_category_list(mc, args={}):
+def do_category_list(mc, args=None):
     """List all available categories."""
+    if args is None:
+        args = {}
     categories = mc.categories.list()
+    _print_category_list(categories)
+
+
+def _print_category_list(categories):
     field_labels = ["ID", "Name"]
     fields = ["id", "name"]
     utils.print_list(categories, fields, field_labels)
@@ -925,8 +1068,8 @@ def do_category_show(mc, args):
            help="Category name.")
 def do_category_create(mc, args):
     """Create a category."""
-    mc.categories.add({"name": args.name})
-    do_category_list(mc)
+    category = mc.categories.add({"name": args.name})
+    _print_category_list([category])
 
 
 @utils.arg("id", metavar="<ID>",
@@ -937,7 +1080,7 @@ def do_category_delete(mc, args):
     for category_id in args.id:
         try:
             mc.categories.delete(category_id)
-        except exceptions.NotFound:
+        except common_exceptions.HTTPNotFound:
             failure_count += 1
             print("Failed to delete '{0}'; category not found".
                   format(category_id))
